"""Add priors

Revision ID: 56177419dc39
Revises: 79137bad215d
Create Date: 2021-04-23 17:56:48.270141

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "56177419dc39"
down_revision = "79137bad215d"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "prior",
        sa.Column("metadata", sa.JSON(), nullable=True),
        sa.Column("pk", sa.Integer(), nullable=False),
        sa.Column("id", postgresql.UUID(), nullable=True),
        sa.Column(
            "time_created",
            sa.DateTime(),
            server_default=sa.text("now()"),
            nullable=True,
        ),
        sa.Column(
            "time_updated",
            sa.DateTime(),
            server_default=sa.text("now()"),
            nullable=True,
        ),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column(
            "function",
            sa.Enum(
                "const",
                "trig",
                "normal",
                "lognormal",
                "truncnormal",
                "stdnormal",
                "uniform",
                "duniform",
                "loguniform",
                "erf",
                "derf",
                name="priorfunction",
            ),
            nullable=False,
        ),
        sa.Column("argument_names", sa.ARRAY(sa.String), nullable=False),
        sa.Column("argument_values", sa.ARRAY(sa.Float), nullable=False),
        sa.Column("experiment_pk", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["experiment_pk"],
            ["experiment.pk"],
        ),
        sa.PrimaryKeyConstraint("pk"),
        sa.UniqueConstraint("id"),
    )
    op.add_column("record", sa.Column("prior_pk", sa.Integer(), nullable=True))
    op.create_foreign_key(None, "record", "prior", ["prior_pk"], ["pk"])
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "record", type_="foreignkey")
    op.drop_column("record", "prior_pk")
    op.drop_table("prior")
    # ### end Alembic commands ###
